@@ -6,12 +6,12 @@
 #modified June 7, 2009 to add gls fitting
 #modified June 24, 2009 to add ml fitting
 "fa" <- 
-function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA,scores=FALSE,SMC=TRUE,missing=FALSE,impute="median", min.err = .001,digits=2,max.iter=50,symmetric=TRUE,warnings=TRUE,fm="minres",...) {
+function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA,scores=FALSE,SMC=TRUE,missing=FALSE,impute="median", min.err = .001,max.iter=50,symmetric=TRUE,warnings=TRUE,fm="minres",...) {
  cl <- match.call()
  control <- NULL   #if you want all the options of mle, then use factanal
  
  ##first some functions that are internal to fa
-  #this does the WLS or ULS fitting  depending upon fm 
+ #this does the WLS or ULS fitting  depending upon fm 
   "fit.residuals" <- function(Psi,S,nf,S.inv,fm) {
               diag(S) <- 1- Psi
               if(!is.null(S.inv)) sd.inv <- diag(1/diag(S.inv))
@@ -21,31 +21,34 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA,scores=FALSE,S
          if(nf >1 ) {loadings <- eigens$vectors[,1:nf] %*% diag(sqrt(eigens$values[1:nf])) } else {loadings <- eigens$vectors[,1] * sqrt(eigens$values[1] ) }
          model <- loadings %*% t(loadings)
        #weighted least squares weights by the importance of each variable   
-       if(fm == "wls" ) {residual <- sd.inv %*% (S- model)^2 %*% sd.inv} else {if (fm=="gls") {residual <- (S.inv %*%(S - model))^2 } else {residual <- (S - model)^2 
+       if(fm == "wls" ) {residual <- sd.inv %*% (S- model)^2 %*% sd.inv} else {if (fm=="gls") {residual <- (S.inv %*%(S - model))^2 } else {residual <- (S - model)^2 #this last is the uls case
        if(fm == "minres") diag(residual) <- 0   #this is minimum residual factor analysis, ignore the diagonal
        }}  # the uls solution usually seems better?
         # 
          error <- sum(residual)
          }
   
- #this code is taken (with minor modification to make ULS, WLS or GLS) from factanal        
+ #this next section is taken (with minor modification to make ULS, WLS or GLS) from factanal        
  #it does the iterative calls to fit.residuals 
  #modified June 7, 2009 to add gls fits
+ #Modified December 11, 2009 to use first derivatives from formula rather than emprical.  this seriously improves the speed.
      "fit" <- function(S,nf,fm) {
           S.smc <- smc(S)
            if((fm=="wls") | (fm =="gls") ) {S.inv <- solve(S)} else {S.inv <- NULL}
-          if(sum(S.smc) == nf) {start <- rep(.5,nf)}  else {start <- 1- S.smc} #initial communality estimates  
-          if(fm=="ml")  {res <- optim(start, FAfn, FAgr, method = "L-BFGS-B",
-                 lower = .005, upper = 1,
-                 control = c(list(fnscale=1,
-                 parscale = rep(0.01, length(start))), control),
-                 nf = nf, S = S)} else {
-          res <- optim(start, fit.residuals, method = "L-BFGS-B", lower = .005, 
-        upper = 1, control = c(list(fnscale = 1, parscale = rep(0.01, 
-            length(start)))), nf= nf, S=S, S.inv=S.inv,fm=fm )}
+           if(sum(S.smc) == nf) {start <- rep(.5,nf)}  else {start <- 1- S.smc} #initial communality estimates  
+           if(fm=="ml")  {res <- optim(start, FAfn, FAgr, method = "L-BFGS-B",
+                          	lower = .005, upper = 1,
+                          	control = c(list(fnscale=1,
+                 			parscale = rep(0.01, length(start))), control),
+                 			nf = nf, S = S)
+                 } else {
+                		 	res <- optim(start, fit.residuals,gr=FAgr.minres, method = "L-BFGS-B", lower = .005, 
+                  			upper = 1, control = c(list(fnscale = 1, parscale = rep(0.01, 
+                  			length(start)))), nf= nf, S=S, S.inv=S.inv,fm=fm )
+                  		}
    
    if((fm=="wls") | (fm=="gls") ) {Lambda <- FAout.wls(res$par, S, nf)} else { Lambda <- FAout(res$par, S, nf)}
-    result <- list(loadings=Lambda,res=res)
+    result <- list(loadings=Lambda,res=res,S=S)
     }
     
  ## the next two functions are taken directly from the factanal function in order to include maximum likelihood as one of the estimate procedures
@@ -57,10 +60,9 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA,scores=FALSE,S
         E <- eigen(Sstar, symmetric = TRUE, only.values = TRUE)
         e <- E$values[-(1:nf)]
         e <- sum(log(e) - e) - nf + nrow(S)
-##        print(round(c(Psi, -e), 5))  # for tracing
-        -e
+       -e
     }
-    FAgr <- function(Psi, S, nf)
+    FAgr <- function(Psi, S, nf)  #the first derivatives
     {
         sc <- diag(1/sqrt(Psi))
         Sstar <- sc %*% S %*% sc
@@ -68,8 +70,20 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA,scores=FALSE,S
         L <- E$vectors[, 1:nf, drop = FALSE]
         load <- L %*% diag(sqrt(pmax(E$values[1:nf] - 1, 0)), nf)
         load <- diag(sqrt(Psi)) %*% load
-        g <- load %*% t(load) + diag(Psi) - S
-        diag(g)/Psi^2
+        g <- load %*% t(load) + diag(Psi) - S     # g <- model - data
+        diag(g)/Psi^2                             #normalized 
+    }
+    
+     FAgr.minres <- function(Psi, S, nf,S.inv,fm)  #the first derivatives 
+    {
+        sc <- diag(1/sqrt(Psi))
+        Sstar <- sc %*% S %*% sc
+        E <- eigen(Sstar, symmetric = TRUE)
+        L <- E$vectors[, 1:nf, drop = FALSE]
+        load <- L %*% diag(sqrt(pmax(E$values[1:nf] - 1, 0)), nf)
+        load <- diag(sqrt(Psi)) %*% load
+        g <- load %*% t(load) + diag(Psi) - S     # g <- model - data
+        diag(g)/Psi^2                             #normalized 
     }
           
  #this was also taken from factanal        
@@ -130,34 +144,40 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA,scores=FALSE,S
     
     #principal axis is an iterative eigen value fitting
     if(fm=="pa") {
-    while(err > min.err)    #iteratively replace the diagonal with our revised communality estimate
-      {
-        eigens <- eigen(r.mat,symmetric=symmetric)
-        #loadings <- eigen.loadings(eigens)[,1:nfactors]
-         if(nfactors >1 ) {loadings <- eigens$vectors[,1:nfactors] %*% diag(sqrt(eigens$values[1:nfactors])) } else {loadings <- eigens$vectors[,1] * sqrt(eigens$values[1] ) }
-         model <- loadings %*% t(loadings)
-         
-         new <- diag(model)
-         
-         comm1 <- sum(new)
-         diag(r.mat) <- new
-         err <- abs(comm-comm1)
-         if(is.na(err)) {warning("imaginary eigen value condition encountered in factor.pa,\n Try again with SMC=FALSE \n exiting factor.pa")
+   	 	e.values <- eigen(r,symmetric=symmetric)$values   #store the original solution
+    	while(err > min.err)    #iteratively replace the diagonal with our revised communality estimate
+     	 {
+       	 eigens <- eigen(r.mat,symmetric=symmetric)
+       	  if(nfactors >1 ) {loadings <- eigens$vectors[,1:nfactors] %*% diag(sqrt(eigens$values[1:nfactors])) } else {loadings <- eigens$vectors[,1] * sqrt(eigens$values[1] ) }
+        	 model <- loadings %*% t(loadings)
+        	 new <- diag(model)       
+         	comm1 <- sum(new)
+         	diag(r.mat) <- new
+        	 err <- abs(comm-comm1)
+        	 if(is.na(err)) {warning("imaginary eigen value condition encountered in factor.pa,\n Try again with SMC=FALSE \n exiting factor.pa")
           break}
-         comm <- comm1
-         comm.list[[i]] <- comm1
-         i <- i + 1
-         if(i > max.iter) {if(warnings)  {message("maximum iteration exceeded")}
-                err <-0 }
+        	 comm <- comm1
+        	 comm.list[[i]] <- comm1
+         	i <- i + 1
+         	if(i > max.iter) { 
+         	         if(warnings)  {message("maximum iteration exceeded")}
+                     err <-0 }
+          }  #end of while loop  
+          eigens <- eigens$values
        } 
-       }
        
        if((fm == "wls") | (fm=="minres") | (fm=="gls") | (fm=="uls")|(fm== "ml")) { 
        uls <- fit(r,nfactors,fm)
        
-       eigens <- eigen(r)  #used for the summary stats
+       e.values <- eigen(r)$values  #eigen values of pc: used for the summary stats --  
        result$par <- uls$res
+      
        loadings <- uls$loadings
+       model <- loadings %*% t(loadings)
+       S <- r
+       diag(S) <- diag(model)   #communalities from the factor model 
+       eigens <- eigen(S)$values
+       
                             }
        
        # a weird condition that happens with the Eysenck data
@@ -182,7 +202,7 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA,scores=FALSE,S
     loadings[loadings==0.0] <- 10^-15    #added to stop a problem with varimax if loadings are exactly 0
    
     model <- loadings %*% t(loadings)  
-   
+    
     f.loadings <- loadings #used to pass them to factor.stats 
     if(rotate != "none") {if (nfactors > 1) {
    
@@ -227,14 +247,15 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA,scores=FALSE,S
    
    result <- factor.stats(r,loadings,Phi,n.obs)   #do stats as a subroutine common to several functions
 
-    result$communality <- round(diag(model),digits)
-    result$uniquenesses <- round(diag(r-model),digits)
-    result$values <- round(eigens$values,digits)
+    result$communality <- diag(model)
+    result$uniquenesses <- diag(r-model)
+    result$values <-  eigens
+    result$e.values <- e.values  
     result$loadings <- loadings
     result$fm <- fm  #remember what kind of analysis we did
-
+    
     if(!is.null(Phi)) {result$Phi <- Phi}
-    if(fm == "pa") result$communality.iterations <- round(unlist(comm.list),digits)
+    if(fm == "pa") result$communality.iterations <- unlist(comm.list)
     
     if(scores) {result$scores <- factor.scores(x.matrix,loadings) }
     result$factors <- nfactors 
