@@ -96,13 +96,16 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA, scores=FALSE,
     result$Phi <- Phi
     result$Call <- cl
     result$communality <- communalities
-    result$stats <- stats
+    #result$stats <- stats
+    result$R2 <- stats$R2
     result$objective <- stats$objective
     result$residual <- stats$residual
     result$fit <-  stats$fit
     result$fit.off <-  stats$fit.off
     result$factors <-  stats$factors
     result$dof <-  stats$dof
+    result$null.dof <- stats$null.dof
+    result$null.model <- stats$null.model
     result$criteria <-  stats$criteria
     result$STATISTIC <-  stats$STATISTIC
     result$PVAL <-  stats$PVAL
